#!/bin/bash

WITH_USR_PART=

FAKE_ROOT=/system

UPI_NO_DELTA_ERROR=fb15
UPI_NO_UA_ERROR=fa17

STATUS_DIR=${FAKE_ROOT}/opt/data/update
STATUS_FILE=${STATUS_DIR}/RW.STATUS
DELTA_PATH_FILE=${STATUS_DIR}/DELTA.PATH
INT_LOG_DIR=${STATUS_DIR}
INT_LOG_FILE=${INT_LOG_DIR}/fota_internal.log

FOTA_LOG_DIR=/opt/data/update
FOTA_RESULT_DIR=${FOTA_LOG_DIR}
FOTA_TEMP_DIR=${FOTA_LOG_DIR}

PART_TBL_FILE="part_tbl.txt"
PART_TBL_PATH=${FOTA_TEMP_DIR}/${PART_TBL_FILE}

CAT="/bin/cat"
SYNC="/bin/sync"
MKDIR="/bin/mkdir"
REBOOT="/sbin/reboot"
MOUNT="/bin/mount"
UMOUNT="/bin/umount"
GREP="/bin/grep"
CUT="/usr/bin/cut"
TR="/usr/bin/tr"
BLKID="/usr/sbin/blkid"
SORT="/usr/bin/sort"
SED="/usr/bin/sed"
WC="/usr/bin/wc"

FOTA_GUI="/usr/bin/fota_gui"
FOTA_GUI_ENABLE=

SYSTEM_DATA_MNT=opt
USER_MNT=opt/usr

#------------------------------------------------
#       get partition id
#------------------------------------------------
get_partition_id() {
	PART_ROOTFS=$("$BLKID" --match-token PARTLABEL=rootfs -o device || "$BLKID" --match-token LABEL=rootfs -o device)
	PART_SYSTEM_DATA=$("$BLKID" --match-token PARTLABEL=system-data -o device || "$BLKID" --match-token LABEL=system-data -o device)
	PART_USER=$("$BLKID" --match-token PARTLABEL=user -o device || "$BLKID" --match-token LABEL=user -o device)
}

#------------------------------------------------
#       check_user_partition
#------------------------------------------------
check_user_partition() {
	RET=$("$CAT" ${FAKE_ROOT}/etc/fstab | "$GREP" "user")
	if [ "z$RET" = "z" ]; then
		return 0
	fi
	return 1
}

#------------------------------------------------
#       mount_partitions
#------------------------------------------------
mount_partitions() {
	get_partition_id

	"$MOUNT" -t ext4 -o ro ${PART_ROOTFS} ${FAKE_ROOT}
	"$MOUNT" -t ext4 ${PART_SYSTEM_DATA} ${FAKE_ROOT}/${SYSTEM_DATA_MNT}

	check_user_partition
	WITH_USR_PART=$?
	if [ "$WITH_USR_PART" = "1" ]; then
		"$MOUNT" -t ext4 ${PART_USER} ${FAKE_ROOT}/${USER_MNT}
	fi

	"$MOUNT" -t proc none ${FAKE_ROOT}/proc
	"$MOUNT" -t sysfs none ${FAKE_ROOT}/sys
	"$MOUNT" -t devtmpfs devtmpfs ${FAKE_ROOT}/dev
	"$MOUNT" -t devpts devpts ${FAKE_ROOT}/dev/pts
}

#------------------------------------------------
#       umount_partitions
#------------------------------------------------
umount_partitions() {

	"$UMOUNT" ${FAKE_ROOT}/dev/pts
	"$UMOUNT" ${FAKE_ROOT}/dev
	"$UMOUNT" ${FAKE_ROOT}/sys
	"$UMOUNT" ${FAKE_ROOT}/proc

	if [ "$WITH_USR_PART" = "1" ]; then
		"$UMOUNT" ${FAKE_ROOT}/${USER_MNT}
	fi
	"$UMOUNT" ${FAKE_ROOT}/${SYSTEM_DATA_MNT}
	"$UMOUNT" ${FAKE_ROOT}
}

#------------------------------------------------
#	handle_no_delta
#------------------------------------------------
handle_no_delta() {
	echo "delta does not exist ..." >> ${INT_LOG_FILE}

	echo "${UPI_NO_DELTA_ERROR}" > ${RESULT_FILE}
}

#------------------------------------------------
#	handle_no_ua
#------------------------------------------------
handle_no_ua() {
	echo "ua does not exist ..." >> ${INT_LOG_FILE}

	echo "${UPI_NO_UA_ERROR}" > ${RESULT_FILE}
}

#------------------------------------------------
#	do_create_part_table
#------------------------------------------------
do_create_part_table() {

	EMMC_DEVICE="/dev/mmcblk0"
	PART_LIST=$("$BLKID" -o full | "$GREP" ${EMMC_DEVICE}p |
			"$SED" -e 's/\/dev\/mmcblk0p//g' -e 's/\ .*LABEL=\"\(.*\).*$/\1/' |
			"$CUT" -d '"' -f 1 | "$SED" 's/\:/\ /' |
			"$TR" '[:upper:]' '[:lower:]' |
			"$SORT" -k 1 -n)
	PART_NUM=$(echo "$PART_LIST" | "$WC" -l)

	echo "$PART_NUM" > ${PART_TBL_PATH}
	echo "$PART_LIST" >> ${PART_TBL_PATH}
	"$SYNC"
}

#------------------------------------------------
#	do_fota_update
#------------------------------------------------
do_fota_update() {
	echo "fota update start ..." >> ${INT_LOG_FILE}

	DELTA_DIR=`cat ${DELTA_PATH_FILE}`
	echo "DELTA_DIR = ${DELTA_DIR}" >> ${INT_LOG_FILE}

	RESULT_FILE=${FAKE_ROOT}${FOTA_RESULT_DIR}/result
	if [ -e ${RESULT_FILE} ]; then
		rm -fr ${RESULT_FILE}
	fi

	UA=${DELTA_DIR}/delta.ua
	if [ ! -s ${UA} ]; then
		handle_no_ua
		return 1
	fi

	/bin/chmod +x ${UA}

	DELTA=${DELTA_DIR}/delta.tar
	if [ ! -s ${DELTA} ]; then
		handle_no_delta
		return 1
	fi

	do_create_part_table
	if [ ! -e ${PART_TBL_PATH} ]; then
		echo "No partition table" >> ${INT_LOG_FILE}
		return 1
	fi

	VERIFY_TOOL=/usr/sbin/upg-verifier
	if [ -e ${VERIFY_TOOL} ]; then
		${VERIFY_TOOL} ${DELTA}
		if [ $? -ne 0 ]; then
			echo "signature verification failed!" >> ${INT_LOG_FILE}
			return 1
		else
			echo "update package verification success"
		fi
	else
		echo "Warning: No signature verifier... Skip it" >> ${INT_LOG_FILE}
	fi

	if [ "$FOTA_GUI_ENABLE" = "1" ]; then
		${UA} ${DELTA_DIR} ${FOTA_TEMP_DIR} 0
	else
		${UA} ${DELTA_DIR} ${FOTA_TEMP_DIR} 1
	fi

	if [ $? -ne 0 ]; then
		echo "update agent fail!!!" >> ${INT_LOG_FILE}
		return 1
	else
		echo "update agent success"
		return 0
	fi

}

#------------------------------------------------
#	init_internal_log
#------------------------------------------------
init_internal_log() {

	if [ -e "${INT_LOG_FILE}" ]; then
		/bin/rm -fr ${INT_LOG_FILE}
	fi
}


#------------------------------------------------
#	init_fota_dir
#------------------------------------------------
init_fota_dir() {

	TEST_DIR=${FAKE_ROOT}${FOTA_LOG_DIR}
	if [ ! -e ${TEST_DIR} ]; then
		"$MKDIR" -p ${TEST_DIR}
		if [ "$?" = "0" ]; then
			echo "success to create ${TEST_DIR}" >> ${INT_LOG_FILE}
		fi
	fi

	TEST_DIR=${FAKE_ROOT}${FOTA_RESULT_DIR}
	if [ ! -e ${TEST_DIR} ]; then
		"$MKDIR" -p ${TEST_DIR}
		if [ "$?" = "0" ]; then
			echo "success to create ${TEST_DIR}" >> ${INT_LOG_FILE}
		fi
	fi

	TEST_DIR=${FAKE_ROOT}${FOTA_TEMP_DIR}
	if [ ! -e ${TEST_DIR} ]; then
		"$MKDIR" -p ${TEST_DIR}
		if [ "$?" = "0" ]; then
			echo "success to create ${TEST_DIR}" >> ${INT_LOG_FILE}
		fi
	fi

}

#------------------------------------------------
#	do_reboot
#------------------------------------------------
do_reboot() {
	echo "Reboot"
	"$SYNC"
	"$REBOOT"
	while [ 1 ]
	do
		sleep 1
		echo "."
	done
}

#------------------------------------------------
#	check_for_rw_power_fail
#------------------------------------------------
check_for_rw_power_fail() {
	# Case : Power fail during RW update after RO update
	if [ -r ${STATUS_FILE} ]; then
		echo "${STATUS_FILE} exists" >> ${INT_LOG_FILE}
		/bin/cat ${STATUS_FILE} >> ${INT_LOG_FILE}
		"$SYNC"
		umount_partitions
		exec /sbin/fus_rw-init
	fi
}

#------------------------------------------------
#	check_for_fota_gui
#------------------------------------------------
check_for_fota_gui() {
	if [ -r ${FOTA_GUI} ]; then
		echo "GUI Enabled" >> ${INT_LOG_FILE}
		FOTA_GUI_ENABLE=1
	fi
}


#------------------------------------------------
#	Main Routine Start
#------------------------------------------------

mount_partitions

# For debugging - It should be deleted on Release (TODO : tty like ttyS0, S1 should be selected automatically)
/sbin/agetty -l /bin/bash -n --keep-baud 115200,38400,9600 ttyS0 linux &

init_internal_log

check_for_rw_power_fail

"$MOUNT" -o remount,rw ${FAKE_ROOT}
init_fota_dir

check_for_fota_gui
if [ "$FOTA_GUI_ENABLE" = "1" ]; then
	export XDG_RUNTIME_DIR=/run
	export TBM_DISPLAY_SERVER=1

	/usr/bin/fota_gui &
	/usr/bin/sleep 2
fi
do_fota_update
fota_result=$?
if [ "$FOTA_GUI_ENABLE" = "1" ]; then
	/usr/bin/fota_gui_test -1
fi
if [ "$fota_result" = "0" ]; then
	/bin/rm -fr ${DELTA_DIR}
fi
umount_partitions

if [ "$fota_result" = "0" ]; then
	exec /sbin/fus_rw-init
else
	echo "Upgrade FAILED"
	do_reboot
fi
